--  Create_Indexes v. 1.0
--
--  This script generates a list (or script, if spooled) of "create
--  index" statements, based on all the indexes in DBA_INDEXES. It must
--  be run under a user with DBA rights, and has been tested under
--  Oracle version 8.1.7. Note that indexes owned by SYS or SYSTEM
--  are excluded from the output, and that the output is sorted by
--  OWNER and INDEX_NAME.
--
--  Limitation: the STORAGE parameters are incomplete for partitioned
--  indexes.
--
--
--                             W. S. Pollack
--                             01/2003
--                             bill@billanddot.com
--                             http://www.billanddot.com/downloads.htm
set verify off
set termout off
set feedback off
set echo off
set pagesize 0
set linesize 100
set long 32767
set trims on
set space 0
set recsep off
set wrap on
--
column nl newline
--
select '-- ================================ --' nl,
       '--   "Create_Indexes" Version 1.0   --' nl,
       '--                                  --' nl,
       '-- ' ||
--           lpad(' ', 6-ceil(length(trim(NAME))/2)) || trim(NAME) ||
             lpad(' ',16-ceil(length(trim(NAME))/2)) || trim(NAME) ||
--           ', ' || to_char(SYSDATE,'MM/DD/YYYY @ HH24:MI') ||
--           lpad(' ', 6-floor(length(trim(NAME))/2)) ||
             lpad(' ',16-floor(length(trim(NAME))/2)) ||
                                          ' --' nl,
       '-- ================================ --' nl,
       ' ' nl
from V$DATABASE
;
--
select
  case
  when COLUMN_POSITION =
       ( select min(COLUMN_POSITION)
         from   DBA_IND_COLUMNS DIC2
         where  DIC2.INDEX_OWNER = DIC.INDEX_OWNER
         and    DIC2.INDEX_NAME = DIC.INDEX_NAME
       )
   and COLUMN_POSITION <
       ( select max(COLUMN_POSITION)
         from   DBA_IND_COLUMNS DIC2
         where  DIC2.INDEX_OWNER = DIC.INDEX_OWNER
         and    DIC2.INDEX_NAME = DIC.INDEX_NAME
       )
  then
       'create' ||
       case
       when INDEX_TYPE = 'BITMAP'
       then ' bitmap '
       when INDEX_TYPE = 'CLUSTER'
       then ' cluster '
       when UNIQUENESS = 'UNIQUE'
       then ' unique '
       else ' '
       end
       || 'index ' ||
       INDEX_OWNER || '.' ||
       DIC.INDEX_NAME || chr(10) ||
       'on          ' || lpad(' ',length(
       case
       when INDEX_TYPE = 'BITMAP'
       then ' bitmap '
       when INDEX_TYPE = 'CLUSTER'
       then ' cluster '
       when UNIQUENESS = 'UNIQUE'
       then ' unique '
       else ' '
       end
       )) ||
       DIC.TABLE_OWNER || '.' || DIC.TABLE_NAME || chr(10) || '(' ||
       rpad(COLUMN_NAME,30) || decode(DESCEND,'DESC',' desc')
  when COLUMN_POSITION >
       ( select min(COLUMN_POSITION)
         from   DBA_IND_COLUMNS DIC2
         where  DIC2.INDEX_OWNER = DIC.INDEX_OWNER
         and    DIC2.INDEX_NAME = DIC.INDEX_NAME
       )
   and COLUMN_POSITION <
       ( select max(COLUMN_POSITION)
         from   DBA_IND_COLUMNS DIC2
         where  DIC2.INDEX_OWNER = DIC.INDEX_OWNER
         and    DIC2.INDEX_NAME = DIC.INDEX_NAME
       )
  then
       ',' ||
       rpad(COLUMN_NAME,30) || decode(DESCEND,'DESC',' desc')
  when COLUMN_POSITION >
       ( select min(COLUMN_POSITION)
         from   DBA_IND_COLUMNS DIC2
         where  DIC2.INDEX_OWNER = DIC.INDEX_OWNER
         and    DIC2.INDEX_NAME = DIC.INDEX_NAME
       )
   and COLUMN_POSITION = 
       ( select max(COLUMN_POSITION)
         from   DBA_IND_COLUMNS DIC2
         where  DIC2.INDEX_OWNER = DIC.INDEX_OWNER
         and    DIC2.INDEX_NAME = DIC.INDEX_NAME
       )
  then
       ',' ||
       rpad(COLUMN_NAME,30) || decode(DESCEND,'DESC',' desc') ||
       chr(10) || ')' || chr(10) ||
       'pctfree ' || PCT_FREE || chr(10) ||
       'initrans ' || INI_TRANS || ' maxtrans ' || MAX_TRANS || chr(10) ||
       decode(LOGGING,'YES','logging','NO','nologging',LOGGING) || chr(10) ||
       'tablespace ' || TABLESPACE_NAME || chr(10) ||
       'storage' || chr(10) || '(' ||
       'buffer_pool ' || lower(BUFFER_POOL) || chr(10) ||
       ' initial ' ||
       case
       when INITIAL_EXTENT/1048576 = round(INITIAL_EXTENT/1048576)
       then INITIAL_EXTENT/1048576 || 'M '
       when INITIAL_EXTENT/1024 = round(INITIAL_EXTENT/1024)
       then INITIAL_EXTENT/1024 || 'K '
       else INITIAL_EXTENT || ' '
       end
       || 'next ' ||
       case
       when NEXT_EXTENT/1048576 = round(NEXT_EXTENT/1048576)
       then NEXT_EXTENT/1048576 || 'M '
       when NEXT_EXTENT/1024 = round(NEXT_EXTENT/1024)
       then NEXT_EXTENT/1024 || 'K '
       else NEXT_EXTENT || ' '
       end
       || 'pctincrease ' || PCT_INCREASE || chr(10) ||
       ' minextents ' || MIN_EXTENTS || ' maxextents ' ||
       decode(max_extents,2147483645,'unlimited',to_char(max_extents)) || chr(10) ||
       ')' || chr(10) ||
       ';'
  else
       'create' ||
       case
       when INDEX_TYPE = 'BITMAP'
       then ' bitmap '
       when INDEX_TYPE = 'CLUSTER'
       then ' cluster '
       when UNIQUENESS = 'UNIQUE'
       then ' unique '
       else ' '
       end
       || 'index ' ||
       INDEX_OWNER || '.' ||
       DIC.INDEX_NAME || chr(10) ||
       'on          ' || lpad(' ',length(
       case
       when INDEX_TYPE = 'BITMAP'
       then ' bitmap '
       when INDEX_TYPE = 'CLUSTER'
       then ' cluster '
       when UNIQUENESS = 'UNIQUE'
       then ' unique '
       else ' '
       end
       )) ||
       DIC.TABLE_OWNER || '.' || DIC.TABLE_NAME || chr(10) || '(' ||
       rpad(COLUMN_NAME,30) || decode(DESCEND,'DESC',' desc') ||
       chr(10) || ')' || chr(10) ||
       'pctfree ' || PCT_FREE || chr(10) ||
       'initrans ' || INI_TRANS || ' maxtrans ' || MAX_TRANS || chr(10) ||
       decode(LOGGING,'YES','logging','NO','nologging',LOGGING) || chr(10) ||
       'tablespace ' || TABLESPACE_NAME || chr(10) ||
       'storage' || chr(10) || '(' ||
       'buffer_pool ' || lower(BUFFER_POOL) || chr(10) ||
       ' initial ' ||
       case
       when INITIAL_EXTENT/1048576 = round(INITIAL_EXTENT/1048576)
       then INITIAL_EXTENT/1048576 || 'M '
       when INITIAL_EXTENT/1024 = round(INITIAL_EXTENT/1024)
       then INITIAL_EXTENT/1024 || 'K '
       else INITIAL_EXTENT || ' '
       end
       || 'next ' ||
       case
       when NEXT_EXTENT/1048576 = round(NEXT_EXTENT/1048576)
       then NEXT_EXTENT/1048576 || 'M '
       when NEXT_EXTENT/1024 = round(NEXT_EXTENT/1024)
       then NEXT_EXTENT/1024 || 'K '
       else NEXT_EXTENT || ' '
       end
       || 'pctincrease ' || PCT_INCREASE || chr(10) ||
       ' minextents ' || MIN_EXTENTS || ' maxextents ' ||
       decode(max_extents,2147483645,'unlimited',to_char(max_extents)) || chr(10) ||
       ')' || chr(10) ||
       ';'
  end
from   DBA_IND_COLUMNS DIC,
       DBA_INDEXES DI
where  INDEX_OWNER = OWNER
and    DIC.INDEX_NAME = DI.INDEX_NAME
and    INDEX_OWNER not in ('SYS','SYSTEM')
order  by OWNER, DI.INDEX_NAME, DIC.COLUMN_POSITION
;
