--  Create_Tables v. 1.0
--
--  This script generates a list (or script, if spooled) of "create
--  table" statements, based on all the tables in DBA_TABLES. It must
--  be run under a user with DBA rights, and has been tested under
--  Oracle version 8.1.7. Note that tables owned by SYS or SYSTEM
--  are excluded from the output, and that the output is sorted by
--  OWNER and TABLE_NAME.
--
--  Parts of this script -- some of the "decode" statements -- are
--  modified from a PL/SQL script by Biju Thomas (see
--  http://www.bijoos.com/oracle/).
--
--  Limitation: the STORAGE parameters are incomplete for partitioned
--  tables.
--
--
--                             W. S. Pollack
--                             01/2003
--                             bill@billanddot.com
--                             http://www.billanddot.com/downloads.htm
set verify off
set termout off
set feedback off
set echo off
set pagesize 0
set linesize 100
set long 32767
set trims on
set space 0
set recsep off
set wrap on
--
column nl newline
--
select '-- ================================ --' nl,
       '--   "Create_Tables" Version 1.0    --' nl,
       '--                                  --' nl,
       '-- ' ||
--           lpad(' ', 6-ceil(length(trim(NAME))/2)) || trim(NAME) ||
             lpad(' ',16-ceil(length(trim(NAME))/2)) || trim(NAME) ||
--           ', ' || to_char(SYSDATE,'MM/DD/YYYY @ HH24:MI') ||
--           lpad(' ', 6-floor(length(trim(NAME))/2)) ||
             lpad(' ',16-floor(length(trim(NAME))/2)) ||
                                          ' --' nl,
       '-- ================================ --' nl,
       ' ' nl
from V$DATABASE
;
--
select
  case
  when COLUMN_ID =
       ( select min(COLUMN_ID)
         from   DBA_TAB_COLUMNS DTC2
         where  DTC2.OWNER = DTC.OWNER
         and    DTC2.TABLE_NAME = DTC.TABLE_NAME
       )
   and COLUMN_ID <
       ( select max(COLUMN_ID)
         from   DBA_TAB_COLUMNS DTC2
         where  DTC2.OWNER = DTC.OWNER
         and    DTC2.TABLE_NAME = DTC.TABLE_NAME
       )
  then
       'create table ' ||
       DTC.OWNER || '.' ||
       DTC.TABLE_NAME || chr(10) || '(' ||
       rpad(COLUMN_NAME,30) || ' ' ||
       decode(DATA_TYPE_OWNER,null,'',DATA_TYPE_OWNER || '.') ||
       rpad(DATA_TYPE,greatest(9,length(DATA_TYPE))) ||
       rpad(decode(DATA_TYPE,'DATE'    ,' '
                            ,'LONG'    ,' '
                            ,'LONG RAW',' '
                            ,'BLOB'    ,' '
                            ,'CLOB'    ,' '
                            ,'ROWID'   ,' '
                            ,'RAW'     ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'CHAR'    ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR' ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR2',decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'NUMBER'  ,decode(data_precision,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
                            ,'FLOAT'  ,decode(DATA_PRECISION,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
               ),8,' ')
  when COLUMN_ID >
       ( select min(COLUMN_ID)
         from   DBA_TAB_COLUMNS DTC2
         where  DTC2.OWNER = DTC.OWNER
         and    DTC2.TABLE_NAME = DTC.TABLE_NAME
       )
   and COLUMN_ID <
       ( select max(COLUMN_ID)
         from   DBA_TAB_COLUMNS DTC2
         where  DTC2.OWNER = DTC.OWNER
         and    DTC2.TABLE_NAME = DTC.TABLE_NAME
       )
  then
       ',' ||
       rpad(COLUMN_NAME,30) || ' ' ||
       decode(DATA_TYPE_OWNER,null,'',DATA_TYPE_OWNER || '.') ||
       rpad(DATA_TYPE,greatest(9,length(DATA_TYPE))) ||
       rpad(decode(DATA_TYPE,'DATE'    ,' '
                            ,'LONG'    ,' '
                            ,'LONG RAW',' '
                            ,'BLOB'    ,' '
                            ,'CLOB'    ,' '
                            ,'ROWID'   ,' '
                            ,'RAW'     ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'CHAR'    ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR' ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR2',decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'NUMBER'  ,decode(data_precision,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
                            ,'FLOAT'  ,decode(DATA_PRECISION,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
               ),8,' ')
  when COLUMN_ID >
       ( select min(COLUMN_ID)
         from   DBA_TAB_COLUMNS DTC2
         where  DTC2.OWNER = DTC.OWNER
         and    DTC2.TABLE_NAME = DTC.TABLE_NAME
       )
   and COLUMN_ID = 
       ( select max(COLUMN_ID)
         from   DBA_TAB_COLUMNS DTC2
         where  DTC2.OWNER = DTC.OWNER
         and    DTC2.TABLE_NAME = DTC.TABLE_NAME
       )
  then
       ',' ||
       rpad(COLUMN_NAME,30) || ' ' ||
       decode(DATA_TYPE_OWNER,null,'',DATA_TYPE_OWNER || '.') ||
       rpad(DATA_TYPE,greatest(9,length(DATA_TYPE))) ||
       rpad(decode(DATA_TYPE,'DATE'    ,' '
                            ,'LONG'    ,' '
                            ,'LONG RAW',' '
                            ,'BLOB'    ,' '
                            ,'CLOB'    ,' '
                            ,'ROWID'   ,' '
                            ,'RAW'     ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'CHAR'    ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR' ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR2',decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'NUMBER'  ,decode(data_precision,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
                            ,'FLOAT'  ,decode(DATA_PRECISION,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
               ),8,' ')
       || chr(10) || ')' || chr(10) ||
       'pctfree ' || PCT_FREE || ' pctused ' || PCT_USED || chr(10) ||
       'initrans ' || INI_TRANS || ' maxtrans ' || MAX_TRANS || chr(10) ||
       decode(LOGGING,'YES','logging','NO','nologging',LOGGING) || chr(10) ||
       'tablespace ' || TABLESPACE_NAME || chr(10) ||
       'storage' || chr(10) || '(' ||
       'buffer_pool ' || lower(BUFFER_POOL) || chr(10) ||
       ' initial ' ||
       case
       when
            INITIAL_EXTENT/1048576 = round(INITIAL_EXTENT/1048576)
       then
            INITIAL_EXTENT/1048576 || 'M '
       when
            INITIAL_EXTENT/1024 = round(INITIAL_EXTENT/1024)
       then
            INITIAL_EXTENT/1024 || 'K '
       else
            INITIAL_EXTENT || ' '
       end
       || 'next ' ||
       case
       when
            NEXT_EXTENT/1048576 = round(NEXT_EXTENT/1048576)
       then
            NEXT_EXTENT/1048576 || 'M '
       when
            NEXT_EXTENT/1024 = round(NEXT_EXTENT/1024)
       then
            NEXT_EXTENT/1024 || 'K '
       else
            NEXT_EXTENT || ' '
       end
       || 'pctincrease ' || PCT_INCREASE || chr(10) ||
       ' minextents ' || MIN_EXTENTS || ' maxextents ' ||
       decode(max_extents,2147483645,'unlimited',to_char(max_extents)) || chr(10) ||
       ')' || chr(10) ||
       ';'
  else
       'create table ' ||
       DTC.OWNER || '.' ||
       DTC.TABLE_NAME || chr(10) || '(' ||
       rpad(COLUMN_NAME,30) || ' ' ||
       decode(DATA_TYPE_OWNER,null,'',DATA_TYPE_OWNER || '.') ||
       rpad(DATA_TYPE,greatest(9,length(DATA_TYPE))) ||
       rpad(decode(DATA_TYPE,'DATE'    ,' '
                            ,'LONG'    ,' '
                            ,'LONG RAW',' '
                            ,'BLOB'    ,' '
                            ,'CLOB'    ,' '
                            ,'ROWID'   ,' '
                            ,'RAW'     ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'CHAR'    ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR' ,decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'VARCHAR2',decode(DATA_LENGTH,null,null
                                                    ,'('||DATA_LENGTH||')')
                            ,'NUMBER'  ,decode(data_precision,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
                            ,'FLOAT'  ,decode(DATA_PRECISION,null,'   '
                                                    ,'('||DATA_PRECISION||
                                 decode(data_scale,0,'',null,null,','||DATA_SCALE)||')')
               ),8,' ')
       || chr(10) || ')' || chr(10) ||
       'pctfree ' || PCT_FREE || ' pctused ' || PCT_USED || chr(10) ||
       'initrans ' || INI_TRANS || ' maxtrans ' || MAX_TRANS || chr(10) ||
       decode(LOGGING,'YES','logging','NO','nologging',LOGGING) || chr(10) ||
       'tablespace ' || TABLESPACE_NAME || chr(10) ||
       'storage' || chr(10) || '(' ||
       'buffer_pool ' || lower(BUFFER_POOL) || chr(10) ||
       ' initial ' ||
       case
       when
            INITIAL_EXTENT/1048576 = round(INITIAL_EXTENT/1048576)
       then
            INITIAL_EXTENT/1048576 || 'M '
       when
            INITIAL_EXTENT/1024 = round(INITIAL_EXTENT/1024)
       then
            INITIAL_EXTENT/1024 || 'K '
       else
            INITIAL_EXTENT || ' '
       end
       || 'next ' ||
       case
       when
            NEXT_EXTENT/1048576 = round(NEXT_EXTENT/1048576)
       then
            NEXT_EXTENT/1048576 || 'M '
       when
            NEXT_EXTENT/1024 = round(NEXT_EXTENT/1024)
       then
            NEXT_EXTENT/1024 || 'K '
       else
            NEXT_EXTENT || ' '
       end
       || 'pctincrease ' || PCT_INCREASE || chr(10) ||
       ' minextents ' || MIN_EXTENTS || ' maxextents ' ||
       decode(max_extents,2147483645,'unlimited',to_char(max_extents)) || chr(10) ||
       ')' || chr(10) ||
       ';'
  end
from   DBA_TAB_COLUMNS DTC,
       DBA_TABLES DT
where  DTC.OWNER = DT.OWNER
and    DTC.TABLE_NAME = DT.TABLE_NAME
and    DTC.OWNER not in ('SYS','SYSTEM')
order  by DT.OWNER, DT.TABLE_NAME, DTC.COLUMN_ID
;
